package com.zhufucdev.motion_emulator

import com.aventrix.jnanoid.jnanoid.NanoIdUtils
import com.zhufucdev.motion_emulator.data.*
import com.zhufucdev.motion_emulator.hook.at
import com.zhufucdev.motion_emulator.hook.center
import kotlinx.serialization.decodeFromString
import kotlinx.serialization.json.Json
import kotlinx.serialization.serializer
import org.junit.Test

import org.junit.Assert.*
import org.junit.Before
import kotlin.math.sqrt
import kotlin.random.Random

class SaltUnitTest {
    private val salts = buildList {
        add(
            Json.decodeFromString(
                serializer<Salt2dData>(),
                """
                    {
                        "id": "mAA1-o_maQO6LEEoApO9-",
                        "elements": [
                          {
                            "values": [
                              "centerX",
                              "centerY"
                            ],
                            "type": "Anchor"
                          },
                          {
                            "values": [
                              "10",
                              "1"
                            ],
                            "type": "Scale"
                          },
                          {
                            "values": [
                              "centerX",
                              "centerY"
                            ],
                            "type": "Anchor"
                          },
                          {
                            "values": [
                              "1",
                              "0"
                            ],
                            "type": "Translation"
                          }
                        ]
                    }
        """
            ).runtime()
        )

        add(
            Json.decodeFromString(
                serializer<Salt2dData>(),
                """
                {
                    "id": "cxvwOuzEtZn5TPpoDBc8r",
                    "elements": [
                      {
                        "values": [
                          "centerX",
                          "centerY"
                        ],
                        "type": "Anchor"
                      },
                      {
                        "values": [
                          "2pi*x",
                          "1"
                        ],
                        "type": "Rotation"
                      }
                    ],
                    "factors": [
                      {
                        "id": "rCWv04jqn-V25ehC1QE4q",
                        "name": "x",
                        "distribution": [
                          0.013320762980579678,
                          0.7749852671986937,
                          0.957235394883454,
                          0.38061790932495787
                        ]
                      }
                    ]
                  }
            """
            ).runtime()
        )
    }
    private val trace by lazy {
        Trace(
            id = NanoIdUtils.randomNanoId(),
            name = "testificate trace",
            listOf(
                Point(-1.0, -1.0),
                Point(-1.0, 1.0),
                Point(1.0, 1.0),
                Point(1.0, -1.0)
            )
        )
    }
    private val trace2 by lazy {
        Json.decodeFromString<Trace>(
            """{"id":"KE4IB0M-HNh2JyA1OeSrY","name":"Near the moon","points":[{"latitude":32.201312412397925,"longitude":118.71214022802896},{"latitude":32.20131127758083,"longitude":118.71213486361101},{"latitude":32.20130560349518,"longitude":118.71211474704376},{"latitude":32.20129312050551,"longitude":118.71208658384958},{"latitude":32.20128971605348,"longitude":118.71207719611819},{"latitude":32.20128517678392,"longitude":118.7120678083868},{"latitude":32.20128063751413,"longitude":118.71206110286438},{"latitude":32.20127723306164,"longitude":118.71205305623747},{"latitude":32.20127382860901,"longitude":118.71204635071506},{"latitude":32.20127042415627,"longitude":118.7120396451926},{"latitude":32.20126815452103,"longitude":118.71203159856572},{"latitude":32.201263615250376,"longitude":118.71202489304328},{"latitude":32.20126021079726,"longitude":118.7120155053119},{"latitude":32.20125567152622,"longitude":118.71200879978947},{"latitude":32.2012499974371,"longitude":118.71199941205808},{"latitude":32.20124545816556,"longitude":118.71199270653565},{"latitude":32.20123978407581,"longitude":118.71198465990874},{"latitude":32.201232975167635,"longitude":118.71197527217736},{"latitude":32.20122616625896,"longitude":118.71196588444597},{"latitude":32.201218222531516,"longitude":118.7119551556101},{"latitude":32.20121141362173,"longitude":118.7119471089832},{"latitude":32.201203469893,"longitude":118.71193906235628},{"latitude":32.20119439134504,"longitude":118.71193101572936},{"latitude":32.20118644761483,"longitude":118.71192431020695},{"latitude":32.20117850388393,"longitude":118.71191894578902},{"latitude":32.201170560152335,"longitude":118.71191224026659},{"latitude":32.201162616420035,"longitude":118.71190687584865},{"latitude":32.20115580750609,"longitude":118.71190151143071},{"latitude":32.201148998591634,"longitude":118.71189614701278},{"latitude":32.20114218967667,"longitude":118.71188944149036},{"latitude":32.20113424594191,"longitude":118.71188407707241},{"latitude":32.20112743702584,"longitude":118.71187871265448},{"latitude":32.20112062810925,"longitude":118.7118760304455},{"latitude":32.201113819192166,"longitude":118.71187066602758},{"latitude":32.20110587545493,"longitude":118.71186530160962},{"latitude":32.201100201356454,"longitude":118.71186127829617},{"latitude":32.20109225761803,"longitude":118.71185725498272},{"latitude":32.201084313878894,"longitude":118.71185457277375},{"latitude":32.20107637013908,"longitude":118.7118505494603},{"latitude":32.201068426398564,"longitude":118.71184652614684},{"latitude":32.201060482657354,"longitude":118.71184384393787},{"latitude":32.20105253891544,"longitude":118.71183982062442},{"latitude":32.20104459517285,"longitude":118.71183713841546},{"latitude":32.201035516609025,"longitude":118.7118317739975},{"latitude":32.20102757286495,"longitude":118.71182909178854},{"latitude":32.201020763940896,"longitude":118.7118250684751},{"latitude":32.20101282019553,"longitude":118.71182238626612},{"latitude":32.20100601127037,"longitude":118.71181836295267},{"latitude":32.200999202344704,"longitude":118.71181568074371},{"latitude":32.20099352823959,"longitude":118.71181433963922},{"latitude":32.200986719312986,"longitude":118.71181165743025},{"latitude":32.20097991038589,"longitude":118.71181031632575},{"latitude":32.20097196663695,"longitude":118.7118076341168},{"latitude":32.20096515770874,"longitude":118.7118062930123},{"latitude":32.20094359609936,"longitude":118.71180226969886},{"latitude":32.20092997823818,"longitude":118.71180226969886},{"latitude":32.20091863001897,"longitude":118.71180092859437},{"latitude":32.200907281798344,"longitude":118.71180092859437},{"latitude":32.2008959335763,"longitude":118.71179958748988},{"latitude":32.200884585352846,"longitude":118.71179824638541},{"latitude":32.20087323712797,"longitude":118.71179824638541},{"latitude":32.20086188890169,"longitude":118.71179824638541},{"latitude":32.20085167549682,"longitude":118.71179690528092},{"latitude":32.20084486655959,"longitude":118.71179556417643},{"latitude":32.20083805762187,"longitude":118.71179422307196},{"latitude":32.200830113860555,"longitude":118.71179288196747},{"latitude":32.20082103527533,"longitude":118.7117901997585},{"latitude":32.20081309151252,"longitude":118.7117901997585},{"latitude":32.200806282572415,"longitude":118.711788858654},{"latitude":32.20079720398482,"longitude":118.71178751754954},{"latitude":32.200785855749054,"longitude":118.71178617644505},{"latitude":32.20077450751186,"longitude":118.71178483534055},{"latitude":32.20076429409719,"longitude":118.71178483534055},{"latitude":32.20075181103325,"longitude":118.71178349423609},{"latitude":32.200740462791806,"longitude":118.71178349423609},{"latitude":32.20072911454895,"longitude":118.7117821531316},{"latitude":32.2007177663047,"longitude":118.7117821531316},{"latitude":32.20070528323437,"longitude":118.7117821531316},{"latitude":32.200693934987136,"longitude":118.7117821531316},{"latitude":32.20068485638833,"longitude":118.71178483534055},{"latitude":32.200675777788625,"longitude":118.71178617644505},{"latitude":32.20066896883824,"longitude":118.71178751754954},{"latitude":32.200661025062146,"longitude":118.71178751754954},{"latitude":32.20065081163472,"longitude":118.711788858654},{"latitude":32.200642867857056,"longitude":118.7117901997585},{"latitude":32.2006349240787,"longitude":118.71179154086299},{"latitude":32.20062698029963,"longitude":118.71179288196747},{"latitude":32.20061790169414,"longitude":118.71179288196747},{"latitude":32.20060882308775,"longitude":118.71179556417643},{"latitude":32.20059974448045,"longitude":118.71179690528092},{"latitude":32.20059066587224,"longitude":118.71179958748988},{"latitude":32.20058158726313,"longitude":118.71180226969886},{"latitude":32.20057023900046,"longitude":118.71180361080334},{"latitude":32.20054640764426,"longitude":118.71180897522129},{"latitude":32.20053052006999,"longitude":118.7118076341168},{"latitude":32.200519171800956,"longitude":118.71181031632575},{"latitude":32.20050555387624,"longitude":118.71180897522129},{"latitude":32.200495340431374,"longitude":118.71181299853474},{"latitude":32.200483992157956,"longitude":118.71181433963922},{"latitude":32.20047264388311,"longitude":118.71181568074371},{"latitude":32.20044540801773,"longitude":118.71181970405716},{"latitude":32.20043405973807,"longitude":118.71182372737061},{"latitude":32.200420441800624,"longitude":118.71182775068408},{"latitude":32.20040682386114,"longitude":118.7118317739975},{"latitude":32.20039547557668,"longitude":118.71183445620649},{"latitude":32.20038753177671,"longitude":118.71183713841546},{"latitude":32.20038072280475,"longitude":118.71183847951995},{"latitude":32.200371644174695,"longitude":118.71184116172891},{"latitude":32.20036256554371,"longitude":118.71184384393787},{"latitude":32.20035121725374,"longitude":118.71184786725134},{"latitude":32.20034100379154,"longitude":118.7118505494603},{"latitude":32.20033192515752,"longitude":118.71185323166928},{"latitude":32.20032171169316,"longitude":118.71185591387824},{"latitude":32.20031376788676,"longitude":118.71185859608721},{"latitude":32.20030355442036,"longitude":118.71186127829617},{"latitude":32.20029447578259,"longitude":118.71186396050516},{"latitude":32.20028426231403,"longitude":118.7118679838186},{"latitude":32.20027631850434,"longitude":118.71187066602758},{"latitude":32.20026723986387,"longitude":118.71187468934103},{"latitude":32.200258161222486,"longitude":118.71187737154999},{"latitude":32.200246812919474,"longitude":118.71188139486345},{"latitude":32.20023886910652,"longitude":118.7118854181769},{"latitude":32.200229790462316,"longitude":118.71188944149036},{"latitude":32.200221846647885,"longitude":118.71189346480382},{"latitude":32.200215037663526,"longitude":118.71189748811724},{"latitude":32.20020709384781,"longitude":118.71190151143071},{"latitude":32.20020141969331,"longitude":118.7119028525352},{"latitude":32.20019574553844,"longitude":118.71190553474416},{"latitude":32.20019007138322,"longitude":118.71190821695312},{"latitude":32.20018326239648,"longitude":118.71190955805761},{"latitude":32.200178723071716,"longitude":118.71191224026659},{"latitude":32.20017418374672,"longitude":118.71191492247557},{"latitude":32.2001696444215,"longitude":118.71191626358004},{"latitude":32.200163970264654,"longitude":118.71191894578902},{"latitude":32.20015943093892,"longitude":118.71192028689349},{"latitude":32.20015489161297,"longitude":118.71192296910246},{"latitude":32.20015035228678,"longitude":118.71192431020695},{"latitude":32.20014467812873,"longitude":118.71192699241591},{"latitude":32.200140138802034,"longitude":118.7119296746249},{"latitude":32.20013446464334,"longitude":118.71193101572936},{"latitude":32.200128790484314,"longitude":118.71193369793833},{"latitude":32.20012198149299,"longitude":118.71193638014732},{"latitude":32.200117442165165,"longitude":118.71193772125179},{"latitude":32.20011290283711,"longitude":118.71193906235628},{"latitude":32.20010722867673,"longitude":118.71194040346077},{"latitude":32.200102689348164,"longitude":118.71194308566973},{"latitude":32.20009701518715,"longitude":118.71194442677422},{"latitude":32.200092475858064,"longitude":118.7119471089832},{"latitude":32.200085666864034,"longitude":118.71194979119215},{"latitude":32.2000811275344,"longitude":118.71195113229665},{"latitude":32.20007658820453,"longitude":118.7119551556101},{"latitude":32.200072048874446,"longitude":118.71195917892354},{"latitude":32.20006297021357,"longitude":118.71196454334148},{"latitude":32.200057296050076,"longitude":118.71196588444597},{"latitude":32.20005275671903,"longitude":118.71196990775944},{"latitude":32.20004708255489,"longitude":118.7119725899684},{"latitude":32.2000414083904,"longitude":118.71197527217736},{"latitude":32.20003686905855,"longitude":118.71197929549082},{"latitude":32.20003119489343,"longitude":118.71198197769978},{"latitude":32.20002665556107,"longitude":118.71198331880427},{"latitude":32.200020981395305,"longitude":118.71198600101323},{"latitude":32.20001644206245,"longitude":118.71198734211772},{"latitude":32.200010767896046,"longitude":118.71199270653565},{"latitude":32.20000509372928,"longitude":118.71199538874464},{"latitude":32.199999419562175,"longitude":118.71199941205808},{"latitude":32.19999488022824,"longitude":118.71200209426706},{"latitude":32.19997445322269,"longitude":118.71201952862535},{"latitude":32.19996877905367,"longitude":118.71202489304328},{"latitude":32.19996197005038,"longitude":118.71203159856572},{"latitude":32.199954026212566,"longitude":118.71204232740159},{"latitude":32.199944947539926,"longitude":118.71205171513297},{"latitude":32.19993132952927,"longitude":118.71206914949127},{"latitude":32.1999199811855,"longitude":118.71208256053613},{"latitude":32.199909767674896,"longitude":118.71209463047647},{"latitude":32.19989955416315,"longitude":118.71210804152133},{"latitude":32.19989047548507,"longitude":118.71211877035721},{"latitude":32.19988366647592,"longitude":118.71212949919308},{"latitude":32.19987572263126,"longitude":118.7121362047155},{"latitude":32.19987231812621,"longitude":118.71214291023793},{"latitude":32.199863239445406,"longitude":118.7121549801783},{"latitude":32.199858700104684,"longitude":118.7121630268052},{"latitude":32.199856430434224,"longitude":118.71217241453658},{"latitude":32.19985189109314,"longitude":118.7121804611635},{"latitude":32.199847351751835,"longitude":118.71218850779042},{"latitude":32.19984281241032,"longitude":118.7121978955218},{"latitude":32.19983940790402,"longitude":118.71220728325316},{"latitude":32.1998348685621,"longitude":118.71221667098457},{"latitude":32.19983259889105,"longitude":118.71222471761149},{"latitude":32.199830329219935,"longitude":118.71223008202942},{"latitude":32.199828059548786,"longitude":118.71223678755183},{"latitude":32.199825789877565,"longitude":118.71224617528324},{"latitude":32.199823520206294,"longitude":118.71225958632807},{"latitude":32.199823520206294,"longitude":118.71226897405946},{"latitude":32.199824655041944,"longitude":118.71227567958191},{"latitude":32.199822385370624,"longitude":118.71228238510432},{"latitude":32.199822385370624,"longitude":118.71228909062674},{"latitude":32.199820115699275,"longitude":118.71229579614915},{"latitude":32.199818980863576,"longitude":118.71230384277607},{"latitude":32.199818980863576,"longitude":118.71231188940298},{"latitude":32.199818980863576,"longitude":118.71231993602987},{"latitude":32.19981784602786,"longitude":118.71232932376128},{"latitude":32.19981784602786,"longitude":118.71233737038818},{"latitude":32.19982125053496,"longitude":118.71234541701507},{"latitude":32.199820115699275,"longitude":118.71235212253752},{"latitude":32.19982125053496,"longitude":118.7123601691644},{"latitude":32.199823520206294,"longitude":118.71236553358236},{"latitude":32.199825789877565,"longitude":118.71237089800027},{"latitude":32.199826924713186,"longitude":118.71238430904515},{"latitude":32.199830329219935,"longitude":118.71239101456756},{"latitude":32.19983259889105,"longitude":118.71240710782136},{"latitude":32.1998360033976,"longitude":118.71241247223931},{"latitude":32.19983940790402,"longitude":118.71242185997069},{"latitude":32.199840542739466,"longitude":118.71242722438863},{"latitude":32.199840542739466,"longitude":118.71243392991106},{"latitude":32.19984281241032,"longitude":118.712439294329},{"latitude":32.199845082081104,"longitude":118.71244599985143},{"latitude":32.199845082081104,"longitude":118.71245404647834},{"latitude":32.199847351751835,"longitude":118.71246075200075},{"latitude":32.19984962142252,"longitude":118.71246611641868},{"latitude":32.19985189109314,"longitude":118.7124728219411},{"latitude":32.19985983493988,"longitude":118.7124996440308},{"latitude":32.19987458779625,"longitude":118.71252914832947},{"latitude":32.19987572263126,"longitude":118.71253585385188},{"latitude":32.1998791271362,"longitude":118.71254121826983},{"latitude":32.199881396806084,"longitude":118.71254658268775},{"latitude":32.19988480131081,"longitude":118.71255194710571},{"latitude":32.199888205815405,"longitude":118.71255731152363},{"latitude":32.19989047548507,"longitude":118.71256267594158},{"latitude":32.19989387998945,"longitude":118.7125680403595},{"latitude":32.19989728449371,"longitude":118.71257340477746},{"latitude":32.19990295866718,"longitude":118.71258413361333},{"latitude":32.199908632840305,"longitude":118.71259352134471},{"latitude":32.19991203734401,"longitude":118.71259754465817},{"latitude":32.199917711516576,"longitude":118.71260559128508},{"latitude":32.19992338568878,"longitude":118.71261497901646},{"latitude":32.19992905986063,"longitude":118.71262436674786},{"latitude":32.19993246436357,"longitude":118.71262973116579},{"latitude":32.19993586886639,"longitude":118.71263509558374},{"latitude":32.19994154303746,"longitude":118.71264314221065},{"latitude":32.19994608237406,"longitude":118.7126471655241},{"latitude":32.199950621710435,"longitude":118.71265252994203},{"latitude":32.19995516104659,"longitude":118.71265789435998},{"latitude":32.199958565548556,"longitude":118.7126645998824},{"latitude":32.199963104884304,"longitude":118.71266996430033},{"latitude":32.199967644219825,"longitude":118.71267532871828},{"latitude":32.199972183555126,"longitude":118.7126806931362},{"latitude":32.19997558805645,"longitude":118.71268605755415},{"latitude":32.199980127391356,"longitude":118.7126900808676},{"latitude":32.19998693639329,"longitude":118.71269812749452},{"latitude":32.199991475727636,"longitude":118.71270215080796},{"latitude":32.2000039588959,"longitude":118.71271690295728},{"latitude":32.200010767896046,"longitude":118.71272092627073},{"latitude":32.20001871172891,"longitude":118.71272629068869},{"latitude":32.20002665556107,"longitude":118.71273165510661},{"latitude":32.20004027355746,"longitude":118.71274104283802},{"latitude":32.20004821738774,"longitude":118.71274506615147},{"latitude":32.20005275671903,"longitude":118.71274506615147},{"latitude":32.20006183538091,"longitude":118.7127504305694},{"latitude":32.20006977920931,"longitude":118.71275311277836},{"latitude":32.20007431853952,"longitude":118.71275445388285},{"latitude":32.2000788578695,"longitude":118.71275713609182},{"latitude":32.20008339719924,"longitude":118.71275847719632},{"latitude":32.20008793652876,"longitude":118.71276115940528},{"latitude":32.200092475858064,"longitude":118.71276250050977},{"latitude":32.20009815001937,"longitude":118.71276518271873},{"latitude":32.200103824180324,"longitude":118.71276652382322},{"latitude":32.200109498340915,"longitude":118.71276920603219},{"latitude":32.200115172501164,"longitude":118.71276920603219},{"latitude":32.20011971182911,"longitude":118.71277188824115},{"latitude":32.20012538598871,"longitude":118.71277457045014},{"latitude":32.20013106014797,"longitude":118.7127759115546},{"latitude":32.2001378691386,"longitude":118.71277859376357},{"latitude":32.20014467812873,"longitude":118.71278127597256},{"latitude":32.2001492174552,"longitude":118.71278261707702},{"latitude":32.200156026444475,"longitude":118.71278529928601},{"latitude":32.20016283543325,"longitude":118.71278529928601},{"latitude":32.200168509590156,"longitude":118.71278664039048},{"latitude":32.200175318578,"longitude":118.71278798149497},{"latitude":32.200186666889905,"longitude":118.71279066370394},{"latitude":32.20019574553844,"longitude":118.71279200480843},{"latitude":32.20021049834035,"longitude":118.7127946870174},{"latitude":32.200221846647885,"longitude":118.7127946870174},{"latitude":32.20023092529289,"longitude":118.7127946870174},{"latitude":32.20023659944557,"longitude":118.7127946870174},{"latitude":32.200244543258705,"longitude":118.71279602812189},{"latitude":32.20025021741053,"longitude":118.71279602812189},{"latitude":32.200258161222486,"longitude":118.71279736922635},{"latitude":32.20026610503375,"longitude":118.71279736922635},{"latitude":32.20027291401428,"longitude":118.71279736922635},{"latitude":32.20028199265419,"longitude":118.71279736922635},{"latitude":32.20028993646337,"longitude":118.71279736922635},{"latitude":32.20029674544212,"longitude":118.71279602812189},{"latitude":32.200304689250004,"longitude":118.71279736922635},{"latitude":32.200311498227656,"longitude":118.71279602812189},{"latitude":32.20031830720478,"longitude":118.71279602812189},{"latitude":32.20032284652259,"longitude":118.71279602812189},{"latitude":32.20032965549888,"longitude":118.7127946870174},{"latitude":32.20033419481611,"longitude":118.7127946870174},{"latitude":32.20033873413312,"longitude":118.7127933459129},{"latitude":32.20034554310822,"longitude":118.7127933459129},{"latitude":32.20035121725374,"longitude":118.71279200480843},{"latitude":32.200358026227896,"longitude":118.71279200480843},{"latitude":32.20036370037264,"longitude":118.71279200480843},{"latitude":32.200370509345866,"longitude":118.71279200480843},{"latitude":32.20037504866107,"longitude":118.71279066370394},{"latitude":32.20038072280475,"longitude":118.71279200480843},{"latitude":32.20038639694808,"longitude":118.71279066370394},{"latitude":32.20039093626249,"longitude":118.71279066370394},{"latitude":32.20039661040518,"longitude":118.71279066370394},{"latitude":32.20040909351785,"longitude":118.71279066370394},{"latitude":32.200413632831136,"longitude":118.71279066370394},{"latitude":32.20041817214418,"longitude":118.71279066370394},{"latitude":32.200423846285176,"longitude":118.71279066370394},{"latitude":32.20042838559772,"longitude":118.71278932259945},{"latitude":32.200432924910025,"longitude":118.71278932259945},{"latitude":32.2004385990501,"longitude":118.71278932259945},{"latitude":32.20044767767348,"longitude":118.71278932259945},{"latitude":32.20045562146819,"longitude":118.71278932259945},{"latitude":32.20046129560685,"longitude":118.71278798149497},{"latitude":32.200470374227976,"longitude":118.71278798149497},{"latitude":32.20047718319321,"longitude":118.71278798149497},{"latitude":32.2004817225031,"longitude":118.71278664039048},{"latitude":32.200488531467485,"longitude":118.71278664039048},{"latitude":32.2004930707768,"longitude":118.71278664039048},{"latitude":32.20049761008588,"longitude":118.71278664039048},{"latitude":32.200503284221924,"longitude":118.71278529928601},{"latitude":32.2005078235305,"longitude":118.71278529928601},{"latitude":32.20051463249294,"longitude":118.71278395818152},{"latitude":32.20052030662791,"longitude":118.71278395818152},{"latitude":32.200524845935654,"longitude":118.71278261707702},{"latitude":32.200531654896814,"longitude":118.71278395818152},{"latitude":32.200539598684195,"longitude":118.71278395818152},{"latitude":32.200549812124095,"longitude":118.71278261707702},{"latitude":32.20055889073638,"longitude":118.71278261707702},{"latitude":32.20056456486859,"longitude":118.71278261707702},{"latitude":32.2005713738268,"longitude":118.71278127597256},{"latitude":32.20057704795822,"longitude":118.71278127597256},{"latitude":32.200582722089315,"longitude":118.71277993486807},{"latitude":32.20058839622005,"longitude":118.71277859376357},{"latitude":32.20059520517645,"longitude":118.71277859376357},{"latitude":32.20060201413236,"longitude":118.71277859376357},{"latitude":32.2006099579136,"longitude":118.71277859376357},{"latitude":32.20061563204262,"longitude":118.7127772526591},{"latitude":32.200625845473986,"longitude":118.7127759115546},{"latitude":32.200638328555215,"longitude":118.71277457045014},{"latitude":32.200646272333294,"longitude":118.71277322934564},{"latitude":32.200651946460056,"longitude":118.71277322934564},{"latitude":32.20065762058647,"longitude":118.71277188824115},{"latitude":32.200663294712534,"longitude":118.71277054713669},{"latitude":32.20067123848842,"longitude":118.71277054713669},{"latitude":32.20067804743863,"longitude":118.71276652382322},{"latitude":32.200685991213234,"longitude":118.71276652382322},{"latitude":32.20069506981193,"longitude":118.71276518271873},{"latitude":32.200701878760356,"longitude":118.71276518271873},{"latitude":32.200709822532865,"longitude":118.71276384161423},{"latitude":32.200716631480205,"longitude":118.71275981830081},{"latitude":32.200722305602575,"longitude":118.71275981830081},{"latitude":32.200730249373315,"longitude":118.71275981830081},{"latitude":32.20073705831911,"longitude":118.71275579498734},{"latitude":32.20074500208857,"longitude":118.71275579498734},{"latitude":32.20075181103325,"longitude":118.7127517716739},{"latitude":32.20075861997742,"longitude":118.7127504305694},{"latitude":32.20076656374499,"longitude":118.7127504305694},{"latitude":32.20077337268807,"longitude":118.71274640725595},{"latitude":32.200781316454346,"longitude":118.71274372504698},{"latitude":32.200788125396315,"longitude":118.71274506615147},{"latitude":32.2007960691613,"longitude":118.71274104283802},{"latitude":32.20080287810218,"longitude":118.71274104283802},{"latitude":32.20080968704253,"longitude":118.71273970173353},{"latitude":32.20081649598239,"longitude":118.71273701952457},{"latitude":32.20082330492173,"longitude":118.7127343373156},{"latitude":32.20083124868364,"longitude":118.7127343373156},{"latitude":32.20083805762187,"longitude":118.71273031400214},{"latitude":32.20084486655959,"longitude":118.71273165510661},{"latitude":32.20085167549682,"longitude":118.71272763179316},{"latitude":32.20085848443352,"longitude":118.71272763179316},{"latitude":32.20086415854705,"longitude":118.7127249495842},{"latitude":32.20087096748283,"longitude":118.71272226737523},{"latitude":32.20088118088553,"longitude":118.71271958516627},{"latitude":32.20089479875403,"longitude":118.71271556185282},{"latitude":32.200903877331875,"longitude":118.71271287964383},{"latitude":32.20091182108676,"longitude":118.71271019743486},{"latitude":32.200917495196975,"longitude":118.7127088563304},{"latitude":32.20092203448488,"longitude":118.71270617412141},{"latitude":32.20092884341632,"longitude":118.7127075152259},{"latitude":32.200933382703674,"longitude":118.71270349191245},{"latitude":32.20093905681252,"longitude":118.71270080970348},{"latitude":32.200945865742696,"longitude":118.71270215080796},{"latitude":32.20095153985078,"longitude":118.71269812749452},{"latitude":32.20095721395851,"longitude":118.71269544528553},{"latitude":32.20096288806589,"longitude":118.71269410418107},{"latitude":32.200968562172896,"longitude":118.71269142197208},{"latitude":32.20097423627957,"longitude":118.7126900808676},{"latitude":32.20097991038589,"longitude":118.71268739865862},{"latitude":32.200986719312986,"longitude":118.71268471644966},{"latitude":32.20099125859745,"longitude":118.71268605755415},{"latitude":32.20099466306064,"longitude":118.7126820342407},{"latitude":32.200999202344704,"longitude":118.7126806931362},{"latitude":32.20100487644946,"longitude":118.7126806931362},{"latitude":32.201009415733004,"longitude":118.71267801092723},{"latitude":32.201015089837135,"longitude":118.71267801092723},{"latitude":32.201020763940896,"longitude":118.71267398761378},{"latitude":32.20102530322366,"longitude":118.71267264650932},{"latitude":32.201029842506195,"longitude":118.71267398761378},{"latitude":32.201035516609025,"longitude":118.71266996430033},{"latitude":32.201040055891056,"longitude":118.71266996430033},{"latitude":32.20104459517285,"longitude":118.71266728209136},{"latitude":32.201053673735764,"longitude":118.7126645998824},{"latitude":32.20105821301688,"longitude":118.71266594098687},{"latitude":32.20106275229777,"longitude":118.71266191767342},{"latitude":32.20106729157843,"longitude":118.71266325877791},{"latitude":32.201071830858865,"longitude":118.71265923546446},{"latitude":32.201079774599094,"longitude":118.71265655325548},{"latitude":32.20108885315849,"longitude":118.71265521215099},{"latitude":32.20109679689721,"longitude":118.71264984773308},{"latitude":32.201101336176194,"longitude":118.71265118883754},{"latitude":32.20110701027458,"longitude":118.7126471655241},{"latitude":32.201111549553026,"longitude":118.71264582441961},{"latitude":32.201116088831256,"longitude":118.71264314221065},{"latitude":32.20112062810925,"longitude":118.71264046000167},{"latitude":32.20112970666458,"longitude":118.71263643668821},{"latitude":32.20114105485746,"longitude":118.71263107227028},{"latitude":32.201145594134225,"longitude":118.71262839006133},{"latitude":32.20115126822984,"longitude":118.71262704895683},{"latitude":32.20115580750609,"longitude":118.71262436674786},{"latitude":32.201162616420035,"longitude":118.71261900232992},{"latitude":32.20116715569573,"longitude":118.71261766122545},{"latitude":32.20117169497117,"longitude":118.71261363791199},{"latitude":32.201176234246404,"longitude":118.71260961459853},{"latitude":32.20118077352141,"longitude":118.71260827349404},{"latitude":32.201185312796184,"longitude":118.71260425018059},{"latitude":32.201189852070726,"longitude":118.71260156797162},{"latitude":32.20119439134504,"longitude":118.71259754465817},{"latitude":32.20119893061914,"longitude":118.71259352134471},{"latitude":32.201206874348266,"longitude":118.71258681582229},{"latitude":32.20121368325839,"longitude":118.71258145140436},{"latitude":32.20121935734977,"longitude":118.71257608698642},{"latitude":32.201227301077104,"longitude":118.712569381464},{"latitude":32.20123637962179,"longitude":118.71256267594158},{"latitude":32.20123978407581,"longitude":118.71255865262812},{"latitude":32.20124432334763,"longitude":118.71255597041916},{"latitude":32.20124772780136,"longitude":118.71255060600122},{"latitude":32.20125567152622,"longitude":118.71254390047879},{"latitude":32.20125907597952,"longitude":118.71253987716534},{"latitude":32.20126701970339,"longitude":118.71253317164292},{"latitude":32.20127042415627,"longitude":118.71252914832947},{"latitude":32.20127382860901,"longitude":118.712525125016},{"latitude":32.20127723306164,"longitude":118.71252110170255},{"latitude":32.2012817723316,"longitude":118.71251573728462},{"latitude":32.20128517678392,"longitude":118.71251037286667},{"latitude":32.20128971605348,"longitude":118.71250098513529},{"latitude":32.20129879459194,"longitude":118.71248355077698},{"latitude":32.20130219904362,"longitude":118.7124741630456},{"latitude":32.20130673831234,"longitude":118.71246477531422},{"latitude":32.20131468203205,"longitude":118.71244599985143},{"latitude":32.201319221300146,"longitude":118.71243661212002},{"latitude":32.20132262575106,"longitude":118.71242588328414},{"latitude":32.20132603020186,"longitude":118.71241649555276},{"latitude":32.20132829983564,"longitude":118.71240308450793},{"latitude":32.20133397391989,"longitude":118.71238430904515},{"latitude":32.20133624355348,"longitude":118.71237089800027},{"latitude":32.20133964800377,"longitude":118.71235882805993},{"latitude":32.201340782820495,"longitude":118.71234675811957},{"latitude":32.201341917637215,"longitude":118.71234139370165},{"latitude":32.201341917637215,"longitude":118.71230518388053},{"latitude":32.201341917637215,"longitude":118.71228909062674},{"latitude":32.201341917637215,"longitude":118.71226092743257},{"latitude":32.20133964800377,"longitude":118.71224349307425},{"latitude":32.201335108736686,"longitude":118.71220862435767},{"latitude":32.20132829983564,"longitude":118.71218448447695},{"latitude":32.201319221300146,"longitude":118.71215900349175}],"salt":{"id":"OeSzZxOsmJqyltSq0GuNw","elements":[{"values":["centerX","centerY"],"type":"Anchor"},{"values":["0.001","0.001"],"type":"Scale"}]}}"""
        )
    }

    @Test
    fun transformers_parsing_correct() {
        val resolution = salts[0].resolve(MapProjector, trace)
        assertEquals("size not match", 2, resolution.size)
        assertEquals("size of transforms not match", 1, resolution[0].transforms.size)
        assertEquals("type of transforms not match", TransformationChain::class, resolution[0].transforms[0]::class)
        // TODO: complete this shit
    }

    @Test
    fun center_correct() {
        val resolution = salts[0].resolve(MapProjector, trace)
        assertEquals("anchor not match", Vector2D.zero, resolution[1].anchor)
    }

    @Test
    fun transform1_correct() {
        val testPoint = Vector2D.one
        val transformed = salts[0].apply(point = testPoint, parent = trace)
        assertEquals("x not match", 11.0, transformed.x, 1e-6)
        assertEquals("y not match", 1.0, transformed.y, 1e-6)
    }

    @Test
    fun transform2_correct() {
        val testPoint = Vector2D.one
        val transformed = salts[1].apply(point = testPoint, parent = trace)
        assertEquals("not in radius", sqrt(2.0), transformed.lenTo(Vector2D.zero), 1e-6)
    }

    @Test
    fun transform3_correct() {
        val center = trace2.center(MapProjector)
        val points = mutableListOf<Vector2D>()
        val references = mutableListOf<Vector2D>()
        repeat(10) {
            val p = Random.nextFloat()
            points.add(trace2.saltedPoints.at(p, MapProjector).point)
            references.add(trace2.points.at(p, MapProjector).point)
        }

        points.forEachIndexed { index, point ->
            val dp = with(MapProjector) { point.distance(center) }
            val dr = with(MapProjector) { references[index].distance(center) * 0.05 }
            assertEquals("far from center", dr, dp, 5.0)
        }
    }
}